@@ -1,20 +1,23 @@
 npudens <-
-  function(bws = stop(paste("bandwidths are required to perform the estimate!",
-             "please set 'bws'")), ...){
-    args = list(...)
-    
-if (is.recursive(bws)){
-    if (!is.null(bws$formula) && is.null(args$tdat))
-      UseMethod("npudens",bws$formula)
-    else if (!is.null(args$data) || !is.null(args$newdata))
-      stop("data and newdata specified, but bws has no formula")
-    else if (!is.null(bws$call) && is.null(args$tdat))
-      UseMethod("npudens",bws$call)
-    else
-      UseMethod("npudens",bws)
-} else {
-      UseMethod("npudens",bws)
-}
+  function(bws, ...){
+    args <- list(...)
+
+    if (!missing(bws)){
+      if (is.recursive(bws)){
+        if (!is.null(bws$formula) && is.null(args$tdat))
+          UseMethod("npudens",bws$formula)
+        else if (!is.null(bws$call) && is.null(args$tdat))
+          UseMethod("npudens",bws$call)
+        else if (!is.call(bws))
+          UseMethod("npudens",bws)
+        else
+          UseMethod("npudens",NULL)
+      } else {
+        UseMethod("npudens", NULL)
+      }
+    } else {
+      UseMethod("npudens", NULL)
+    }
   }
 
 npudens.formula <-
@@ -164,18 +167,60 @@ npudens.bandwidth <-
   return(ev)
 }
 
-npudens.default <-
-  function(bws,
-           tdat = stop("invoked without training data 'tdat'"),
-           edat, ...){
-    tdat <- toFrame(tdat)
-    
-    tbw <- npudensbw(bws = bws,
-                     dat = tdat,
-                     bandwidth.compute = FALSE,
-                     ...)
+npudens.default <- function(bws, tdat, ...){
+  sc.names <- names(sys.call())
 
-    eval(parse(text=paste("npudens.bandwidth(tdat = tdat, bws = tbw",
-                 ifelse(missing(edat), "",", edat = edat"), ")")))
+  ## here we check to see if the function was called with tdat =
+  ## if it was, we need to catch that and map it to dat =
+  ## otherwise the call is passed unadulterated to npudensbw
+
+  bws.named <- any(sc.names == "bws")
+  tdat.named <- any(sc.names == "tdat")
+
+  no.bws <- missing(bws)
+  no.tdat <- missing(tdat)
+
+  ## if bws was passed in explicitly, do not compute bandwidths
+    
+  if(tdat.named)
+    tdat <- toFrame(tdat)
+
+  mc <- match.call()
+  
+  tbw <- eval(parse(text = paste("npudensbw(",
+                      ifelse(tdat.named, "dat = tdat",
+                             ifelse(no.tdat,"","tdat")),
+                      ifelse(no.tdat,"",","),
+                      ifelse(bws.named,"bws = bws, bandwidth.compute = FALSE",
+                             ifelse(no.bws,"","bws")),
+                      ifelse(no.bws,"",","),                      
+                      "call = mc, ...",")",sep="")))
+
+  ## need to do some surgery on the call to
+  ## allow it to work with the formula interface
+
+  repair.args <- c("data", "subset", "na.action")
+  
+  m.par <- match(repair.args, names(mc), nomatch = 0)
+  m.child <- match(repair.args, names(tbw$call), nomatch = 0)
+
+  if(any(m.child > 0)) {
+    tbw$call[m.child] <- mc[m.par]
   }
 
+  ## next we repair 'bws' portion of the call
+  m.bws.par <- match(c("bws","tdat"), names(mc), nomatch = 0)
+  m.bws.child <- match(c("bws","tdat"), as.character(tbw$call), nomatch = 0)
+  m.bws.union <- (m.bws.par > 0) & (m.bws.child > 0)
+  
+  tbw$call[m.bws.child[m.bws.union]] <- mc[m.bws.par[m.bws.union]]
+  
+  environment(tbw$call) <- parent.frame()
+
+  ## convention: first argument is always dropped, second, if present, propagated
+  eval(parse(text=paste("npudens(bws = tbw",
+               ifelse(no.tdat, "",
+                      ifelse(tdat.named, ",tdat = tdat",",tdat")),
+               ",...)")))
+}
+
